﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using Sitecore.Shell.Framework.Commands;
using Sitecore.Web.UI.Sheer;
using Sitecore.Diagnostics;
using Sitecore.Data.Items;
using Sitecore;
using System.Collections.Specialized;
using Sitecore.Resources.Media;
using nQuant;
using System.Drawing;
using System.Drawing.Imaging;
using Sitecore.Globalization;
using System.IO;
using Sitecore.SecurityModel;
using Sitecore.Data.Managers;
using Sitecore.Shell.Framework;
using Sitecore.Data;
using Sitecore.Configuration;
using System.Configuration;

namespace RC.SCModule.PNGOptimizer.Commands
{
    public class Optimize : Command
    {
        public override void Execute(CommandContext context)
        {
            Assert.ArgumentNotNull(context, "context");

            if ((int)context.Items.Length == 1)
            {
                Item mainItem = context.Items[0];
                NameValueCollection nameValueCollection = new NameValueCollection();
                string[] item = new string[] { context.Parameters["id"], mainItem.ID.ToString() };
                nameValueCollection["id"] = StringUtil.GetString(item);
                nameValueCollection["language"] = mainItem.Language.ToString();
                nameValueCollection["version"] = mainItem.Version.ToString();
                string mode = context.Parameters["mode"];
                if (String.IsNullOrEmpty(mode))
                    Context.ClientPage.Start(this, "Run", nameValueCollection);
                else if (mode == "one")
                    Context.ClientPage.Start(this, "RunOne", nameValueCollection);

                return;
            }
            else
            {
                return;
            }
        }

        public void RunOne(ClientPipelineArgs args)
        {
            Assert.ArgumentNotNull(args, "args");
            string itemId = args.Parameters["id"];
            string lang = args.Parameters["language"];
            string ver = args.Parameters["version"];

            Item itm = Client.ContentDatabase.Items[itemId, Language.Parse(lang), Sitecore.Data.Version.Parse(ver)];
            Media mi = Sitecore.Resources.Media.MediaManager.GetMedia(new MediaItem(itm));

            if (mi.Extension.ToLower() != "png")
            {
                Context.ClientPage.ClientResponse.Alert(Translate.Text("The image \"{0}\" is not a PNG image file.", itm.Name));
                return;
            }

            itm.BeginEdit();


            if (mi.Extension.ToLower() == "png") // || mi.Extension.ToLower() == "jpg" || mi.Extension.ToLower() == "gif")
            {
                var quantizer = new WuQuantizer();

                using (var bitmap = new Bitmap(GetPngStream(mi), false))
                {
                    using (var quantized = quantizer.QuantizeImage(bitmap))
                    {
                        System.IO.MemoryStream ms = new System.IO.MemoryStream();
                        ImageFormat format = ImageFormat.Png;
                        quantized.Save(ms, format);

                        mi.SetStream(ms, "png");
                    }

                }
            }
            itm.EndEdit();

            Context.ClientPage.SendMessage(this, "media:refresh");

            return;
        }

        public void Run(ClientPipelineArgs args)
        {
            Assert.ArgumentNotNull(args, "args");
            bool isPostbBck = args.IsPostBack;
            Item destinationItem = null;

            if (isPostbBck)
            {
                string itemId = args.Parameters["id"];
                string lang = args.Parameters["language"];
                string ver = args.Parameters["version"];
                string alphaThresholdVal = Sitecore.Configuration.Settings.GetSetting("alphaThreshold", "10");
                string alphaFaderVal = Sitecore.Configuration.Settings.GetSetting("alphaFader", "70");
                int alphaThreshold = 10;
                int alphaFader = 70;

                int.TryParse(alphaThresholdVal, out alphaThreshold);
                int.TryParse(alphaFaderVal, out alphaFader);

                Item mainItem = Client.ContentDatabase.Items[itemId, Language.Parse(lang), Sitecore.Data.Version.Parse(ver)];


                foreach (Item itm in mainItem.GetChildren())
                {
                    itm.BeginEdit();

                    Media mi = Sitecore.Resources.Media.MediaManager.GetMedia(new MediaItem(itm));

                    if (mi.Extension.ToLower() == "png")
                    {
                        var quantizer = new WuQuantizer();

                        using (var bitmap = new Bitmap(GetPngStream(mi), false))
                        {
                            using (var quantized = quantizer.QuantizeImage(bitmap, alphaThreshold, alphaFader))
                            {
                                System.IO.MemoryStream ms = new System.IO.MemoryStream();
                                ImageFormat format = ImageFormat.Png;
                                quantized.Save(ms, format);
                                if (String.IsNullOrEmpty(args.Result) || args.Result == "null" || args.Result == mainItem.Paths.ContentPath)
                                {
                                    //if (mi.Extension.ToLower() != "png")
                                    //{
                                    //    using (new SecurityDisabler())
                                    //    {
                                    //        itm.ChangeTemplate(new TemplateItem(Context.Database.GetItem(TemplateIDs.UnversionedImage)));
                                    //    }
                                    //}

                                    mi.SetStream(ms, "png");
                                }
                                else
                                {
                                    MediaCreatorOptions options = new MediaCreatorOptions();
                                    options.FileBased = false;
                                    options.Versioned = false;

                                    destinationItem = Context.Database.GetItem(args.Result);

                                    if (destinationItem == null)
                                        destinationItem = mainItem;

                                    options.Destination = destinationItem.Paths.FullPath + "/" + itm.Name;

                                    Item tmpItem = Context.Database.GetItem(options.Destination);

                                    if (tmpItem != null && tmpItem.ID == itm.ID)
                                    {
                                        mi.SetStream(ms, "png");
                                    }
                                    else
                                    {
                                        MediaManager.Creator.CreateFromStream(ms, itm.Name + ".png", options);
                                    }
                                }
                            }

                        }
                    }
                    itm.EndEdit();
                }

                if (destinationItem != null)
                {
                    string str = string.Format("item:load(id={0},language={1},version={2})", destinationItem.ID.ToString(), destinationItem.Language.ToString(), Sitecore.Data.Version.Latest.ToString());
                    Context.ClientPage.SendMessage(this, str);
                }

                return;
            }
            else
            {
                //SheerResponse.Input("Enter output path:", "");

                string item = args.Parameters["id"];
                if (item != null)
                {
                    Dialogs.BrowseItem("Destination item.", "Select the item to save the optimized images to.", "Core3/16x16/open_document.png", "Select", "/sitecore/media library/", string.Concat(item, "/"));
                    args.WaitForPostBack();
                }
                else
                {
                    return;
                }
            }
        }

        private Stream GetPngStream(Media mediaItem)
        {
            Stream newStream = mediaItem.GetStream().Stream;
            if (mediaItem.Extension.ToLower() == "jpg" || mediaItem.Extension.ToLower() == "gif")
            {
                Image bmpImageToConvert = Image.FromStream(mediaItem.GetStream().Stream);
                Image bmpNewImage = new Bitmap(bmpImageToConvert.Width,
                                       bmpImageToConvert.Height);
                Graphics gfxNewImage = Graphics.FromImage(bmpNewImage);
                gfxNewImage.DrawImage(bmpImageToConvert,
                                      new Rectangle(0, 0, bmpNewImage.Width,
                                                    bmpNewImage.Height),
                                      0, 0,
                                      bmpImageToConvert.
                                      Width, bmpImageToConvert.Height,
                                      GraphicsUnit.Pixel);
                gfxNewImage.Dispose();
                bmpImageToConvert.Dispose();

                MemoryStream newImageStream = new MemoryStream();
                bmpNewImage.Save(newImageStream, ImageFormat.Png);
                newStream = newImageStream;
            }

            return newStream;
        }
    }
}